/***********************************************************************************************
 * AnalysisMain.hpp                                                                           *
 *                                                                                             *
 *  This class controls the steering of the analysis.                                          *
 *                                                                                             *
 * Public Member Functions of AnalysisMain class                                              *
 *    AnalysisMain()        -- Default Constructo                                             *
 *    ~AnalysisMain()       -- Destructor                                                     *
 *    OpenHistogramFile()    -- Open file for output histograms                                *
 *    WriteHistogramFile()   -- Write output histograms                                        *
 *    CloseHistogramFile()   -- Close output histogram file                                    *
 *    GetHistogramFileName() -- Get file name                                                  *
 *    Loop()                 -- Loop over events                                               *
 *    BookHistos()           -- Not needed for this class                                      *
 *    Apply()                -- Not needed for this class                                      *
 *                                                                                             *
 * Private Data Members                                                                        *
 *    TFile *_histoFile      -- Histogram File                                                 *
 *    string _histoFileName  -- Name of the histogram file                                     *
 *    int _numEvents         -- Number of Events                                               *
 *                                                                                             *
 **********************************************************************************************/

#ifndef AnalysisMain_h
#define AnalysisMain_h

#include <string>

#include "TFile.h"
// 
#include "single_top_analysis/Particles.hpp"
#include "single_top_analysis/CutListProcessor.hpp"


class AnalysisMain: public Particles, public CutListProcessor
{
 public:
  // Default Constructor
  AnalysisMain();
  // Destructor
  ~AnalysisMain();

  // Open the output histogram file
  inline void OpenHistogramFile(const std::string& name) {
    _histoFile = new TFile(name.c_str(), "RECREATE"); 
    _histoFileName = name; 
  }

  // Write histograms to open output file
  inline void WriteHistogramFile() { 
    if(_histoFile) _histoFile->Write(); 
  }

  // Close the output histogram file
  inline void CloseHistogramFile() { 
    if (_histoFile) {
     _histoFile->Close(); 
     delete _histoFile; 
     _histoFile=NULL; 
    } //if
  }

  // Get the name of the histogram file
  inline std::string GetHistogramFileName() { return _histoFileName; };
  
  //
  // same thing for the skim file
  // Open the output skim file
  inline void OpenSkimFile(const std::string& name) {
    _skimFile = new TFile(name.c_str(), "RECREATE"); 
    _skimFileName = name; 
  }

  // Write histograms to open output file
  inline void WriteSkimFile() { 
    if(_skimFile) { _skimFile=_skimCoTr->GetCurrentFile(); _skimFile->Write(); }
  }

  // Close the output histogram file
  inline void CloseSkimFile() { 
    if (_skimFile) {
      _skimFile=_skimCoTr->GetCurrentFile(); 
      _skimFile->Close(); 
     delete _skimFile; 
     _skimFile=NULL; 
    } //if
  }

  // Get the name of the skim file
  std::string GetSkimFileName() { return _skimFileName; };
  
  // Parse the command line to extract the names of 
  // the root files and put them into chains.
  Int_t ParseCmdLine(int argc, char **argv,TChain *coch,TChain *evch,TChain *trch);

  // 
  // Loop over events
  void Loop();
  //
  // Declare these two because they are virtual in the class we are inheriting from
  // they are not needed for AnalysisMain class
  void BookHistos(void) {};
  bool Apply() { return true; };
 
 private:  
  TFile *_histoFile;            // Output histogram file
  std::string _histoFileName;   // Output histogram file name
  int _numEvents;               // Number of events to loop over (set to 0 to loop over all events)

  Bool_t _doSkim;              // Should we write out selected events to a new skim?        
  TFile *_skimFile;            // Output skim file
  std::string _skimFileName;   // Name of the skim file
  TTree* _skimCoTr;            // Collection tree in the skim file
  TTree* _skimEvTr;            // Event tree in the skim file
  TTree* _skimTrTr;            // Truth tree in the skim file
  


  ////////////////////////////////////////
  // needed to run with Root:
  ClassDef(AnalysisMain,0)
};


#endif
