/////////////////////////////////////////////////////////////////////////////////
// file: AnalysisMain
// 
// Description: main class controlling the steering of the single top analysis.
//
// Author: Reinhard Schwienhorst
//
// Date: 04/07/2005
//
// Modifications:
//
/////////////////////////////////////////////////////////////////////////////////
#include "single_top_analysis/AnalysisMain.hpp"
#include <iostream>
#include <string>
#include <fstream>

#include <TStopwatch.h>
#include <TTreeIndex.h>
#include <TSystem.h>

#include "single_top_analysis/utils.hpp"
//

using namespace std;

// needed to run with compiled Root:
ClassImp(AnalysisMain)


//
// this is where the action is:
void AnalysisMain::Loop()
{
  // book histograms
  SetDirectory (_histoFile);
  CutListProcessor::BookHistos();
  //
  // now start the loop over events 
  // first over the root file without gluons:
  bool limitEvents= (_numEvents>0);

  // start the stopwatch
  TStopwatch w;
  w.Start();
  
  int iEvt=GetNextEvent();
  int nevents=0;

  Bool_t writeThisEvent=kFALSE;
  Bool_t firstEventWritten=kFALSE;

  // either loop until we are out of events or until we have reached
  // the limit specified on the command line
  //cout<<"Beginning loop over events, _numEvent is "<<_numEvents<<endl;
  while(iEvt != -1 && (!limitEvents || iEvt<_numEvents)) {  
    nevents++;
    if(iEvt%10000==0) cout<<"Processing event "<<iEvt<<endl;

    //evtNum=iEvt;
    //
    //
    // this is from CutListProcessor:
    // now fill the histograms
    //cout<<"calling apply"<<endl;
    writeThisEvent = CutListProcessor::Apply(this);

    //
    // if this event passes all tasks, write it out
    //
    
    if(_doSkim && writeThisEvent) {
      if (!firstEventWritten) {
	firstEventWritten = kTRUE;
	//add_skim_branches();
      }
      if(_skimCoTr!=NULL) _skimCoTr->Fill();
      if(_skimEvTr!=NULL) _skimEvTr->Fill();
      if(_skimTrTr!=NULL) _skimTrTr->Fill();
    }
    

    //
    // and read the next event
    //cout<<"Getting next event"<<endl;
    iEvt=GetNextEvent();
  } // end of loop!

  cout<<endl<<"Done processing "<<nevents<<" events."<<endl<<endl;
  // stop the timer and print out timing statistics
  w.Stop();
  Double_t time_spent=w.RealTime();
  Double_t time_event=time_spent/(Double_t)nevents;

  cout<<"Timing info:"<<endl;
  cout<<"  Real time (s):      "<<time_spent<<endl;
  cout<<"  Time per event (s): "<<time_event<<endl;
  cout<<"  Events/second       "<<1./time_event<<endl;
  cout<<endl;


  // writing histogram file and close it
  WriteHistogramFile();
  CloseHistogramFile();
  
  // writing skim file and close it if necessary
  if(_doSkim) {
    WriteSkimFile();
    CloseSkimFile();
  }

} 

/////////////////////////////////////////////////////////////////////
// constructor:
AnalysisMain::AnalysisMain(): CutListProcessor("cutter")
			      , _histoFile(NULL),_numEvents(0),_histoFileName("dummy.root")
			      ,_skimFileName("NONE"),_doSkim(kFALSE)
			      ,_skimCoTr(NULL),_skimEvTr(NULL),_skimTrTr(NULL)
{
  //cout<<"AnalysisMain::AnalysisMain() constructor"<<endl;
}

////////////////////////////////////////////////////////////////////////
// destructor:
AnalysisMain::~AnalysisMain()
{
}

////////////////////////////////////////////////////////////////////////////////////////////
//
// function to parse the command line and extract
// the name of the top root file and trigger root file.
Int_t AnalysisMain::ParseCmdLine(int argc, char **argv,TChain *coch,TChain *evch,TChain *trch)
{
  if(argc<3) {
    cout<<"**********************************************************************"<<endl;
    cout<<"* Program: AnalysisMain                                             *"<<endl;
    cout<<"*                                                                    *"<<endl;
    cout<<"* Purpose: analyze a parton level single top root file.              *"<<endl;
    cout<<"*                                                                    *"<<endl;
    cout<<"* Usage: AnalysisMain -inlist filename                              *"<<endl;
    cout<<"*  You need to specify a root file or list of files                  *"<<endl;
    cout<<"*                                                                    *"<<endl;
    cout<<"*  -inlist file    - open a file with a list of root files.          *"<<endl;
    cout<<"*  -infile file    - load a single root file.                        *"<<endl;
    cout<<"*                                                                    *"<<endl;
    cout<<"*  -config cfgfile - use this configuration file.                    *"<<endl;
    cout<<"*                                                                    *"<<endl;
    cout<<"*  -source name    - name of the source we are processing.           *"<<endl;
    cout<<"*                    This can also be passed through the list file.  *"<<endl;
    cout<<"*  -num #          - number of events to process.                    *"<<endl;
    cout<<"*  -offSet #       - start processing not at 0 but here.             *"<<endl;
    cout<<"*  -hfile file     - filename for output histograms.                 *"<<endl;
    cout<<"*  -skimfile file  - filename to write selected events to.           *"<<endl;
    cout<<"*                                                                    *"<<endl;
    cout<<"**********************************************************************"<<endl;
    return 1;
  }

 
  cout<<"AnalysisMain: Parsing command line."<<endl;

  // check that we have valid chain pointers:
  if(coch==0) {
    cout<<"<AnalysisMain::ParseCmdLine> No valid input Collection Chain."<<endl;
    return -1;
  }

  // check that we have valid chain pointers:
  if(evch==0) {
    cout<<"<AnalysisMain::ParseCmdLine> No valid input Event Chain."<<endl;
    return -1;
  }

  // check that we have valid chain pointers:
  if(trch==0) {
    cout<<"<AnalysisMain::ParseCmdLine> No valid input Truth Chain."<<endl;
    //cout<<"This is ok for now..."<<endl;
    return -1;
  }

  // default histogram file name:
  std::string histoFileName="Histograms.root";

  CollectionTree* treeCollection=0;
  EventTree* treeEventTree=0;
  TruthTree* treeTruthTree=0;

  int evtOffSet=0;

  // Variables for IO
  int iLOList = 0; // Index of pointer to file-list name in arguments
  int iLOFile = 0; // Index of pointer to file name in arguements
  bool useLOFile = false;
  bool useLOList = false;

  //
  // save the current path to get back to it later
  string cwd=gSystem->pwd();
  cwd+="/";
  
  // Parse Command Line for 
  for (int i = 1; i < argc; ++i) {
    //cout<<"Command line parameter "<<i<<" is "<<argv[i]<<endl;
    //
    // check if this is the flag for a file with a list of root files
    if (!strcmp(argv[i],"-inlist")) {
      // 
      if (argc < i+1) {
	cout << "<Error> Missing file-list name!" << endl;
	return 1;
      }
      iLOList = i+1;
      ++i;
      useLOList = true;
    } 
    else if (!strcmp(argv[i],"-infile")) {  // Get name of file
      // 
      // check if this is a single root file
      if (argc < i+1) {
	cout << "<Error> Missing input-file name!" << endl;
	return 1;
      }//if
      iLOFile = i+1;
      ++i;
      useLOFile = true;
    }//elseif this is a single file
    else if (!strcmp(argv[i],"-config")) {  // Get name of file
      // 
      // check if the name of the configuration file is given properly
      if (argc < i+1) {
	cout << "<Error> Missing configuration file name!" << endl;
	return 1;
      }//if
      TString confFile=cwd + argv[i+1];
      // check if this input file exists!
      FILE *ifp;
      if(!(ifp = fopen(confFile.Data(), "r"))) {
	cout<<"ERROR: input configuration file "<<confFile.Data()<<" does not exist!"<<endl;
	return -1;
      }
      fclose(ifp);
      _config.ReadFile(confFile.Data(), kEnvChange);
      ++i;
    }//elseif configuration file name
    else if (!strcmp(argv[i],"-source")) {  // name of the source that we are reading
      //
      if (argc < i+1) {
	cout << "<Error> Missing Value for source name -source!" << endl;
	return 1;
      }
      SetSourceName(argv[i+1]);
      ++i;
    } //elseif source
    else if (!strcmp(argv[i],"-num")) {  // number of events
      //
      if (argc < i+1) {
	cout << "<Error> Missing Value for number of events -num!" << endl;
	return 1;
      }
      _numEvents=atoi(argv[i+1])+evtOffSet;
      ++i;
    } //elseif num events
    else if (!strcmp(argv[i],"-offSet")) {  // number of events
      //
      if (argc < i+1) {
	cout << "<Error> Missing Value for number of events -offSet!" << endl;
	return 1;
      }
      evtOffSet=atoi(argv[i+1]);
      if(_numEvents>0) _numEvents+=evtOffSet;
      ++i;
    } //elseif offSet events
    else if (!strcmp(argv[i],"-hfile")) {  // output histogram file
      //
      if (argc < i+1) {
	cout << "<Error> Missing Value for histogram filename -hfile!" << endl;
	return 1;
      }
      histoFileName=argv[i+1];
      ++i;
    } //elseif histogram file name
    else if (!strcmp(argv[i],"-skimfile")) {  // output skim file
      //
      if (argc < i+1) {
	cout << "<Error> Missing Value for skim filename -skimfile!" << endl;
	return 1;
      }
      _doSkim=true;
      _skimFileName=argv[i+1];
      ++i;
    } //elseif skim file name
  }//for loop over command line arguments
  
  bool doPathWarning=true;
  //////////////////////////////////////////////
  //
  // Make the chain for the input files.  Use filelist if we have one, else add input file
  // to chain, else exit
  if (useLOList) // Chain files from input filelist
    {
      // Use fstream
      ifstream in(argv[iLOList]);
      int nfiles = 0;
      
      if (in) { // file with list exists
	string fileName,fileOnly,pathOnly;
	int ichain;
	bool foundSourceName=false;
	
	cout << "<AnalysisMain> Chaining files listed in ";
	cout << argv[iLOList] << ":" << endl;
	while (getline(in, fileName)) {
	  // check for comment lines
	  if(fileName.find("#")!=string::npos) continue;
	  // the first line that is not a comment line contains the source name
	  if(!foundSourceName) {
	    cout << "<AnalysisMain> Setting source name to: "<<fileName.c_str()<<endl;
	    SetSourceName(fileName.c_str());
	    foundSourceName=true;
	    continue;
	  }
	  cout << "<AnalysisMain> reading root tree from " << fileName.c_str() << endl;
	  // we have to do some gymnastics here, 
	  // otherwise loading the tree will produce an error:
	  // strip off the path first
	  string::size_type pos = fileName.rfind("/");
	  fileOnly=fileName.substr(pos+1);
	  pathOnly=fileName.substr(0,pos);
	  if(doPathWarning) {
	    cout<<"<AnalysisMain> Note that these trees don't load properly if they aren't all "<<endl;
	    cout<<"in the same directory.We need to switch into that directory now and then start"<<endl;
	    cout<<"reading things in."<<endl;
	    cout<<"found path: "<<pathOnly<<endl;
	    cout<<"and file "<<fileOnly<<endl;
	    doPathWarning=false;
	  }
	  gSystem->cd(pathOnly.c_str());
	  ichain=coch->Add(fileOnly.c_str());
	  evch->Add(fileOnly.c_str());
	  trch->Add(fileOnly.c_str());
	  if (ichain) ++nfiles;
	}
	cout << "<AnalysisMain> " << nfiles << " files successfully chained." ;
	cout << endl;
	if (nfiles==0) {
	  cout << "Exiting." << endl;
	  return 1;
	}
      }//if
   } 
  else if (useLOFile) { // Run on input file
    int ichain;
    cout << "<AnalysisMain> Will run on " << argv[iLOFile] << endl;
    ichain=coch->Add(argv[iLOFile]);
    evch->Add(argv[iLOFile]);
    trch->Add(argv[iLOFile]);
    if (!ichain) {
      cout << "<AnalysisMain> Error chaining " << argv[iLOFile];
      cout << "! Exiting." << endl;
      return 1;
    }
  } 
  else {
    // Here we don't have any inputs so exit with error.
    cout << "<AnalysisMain> Need input LO root file/filelist." << endl;
    return 1;
  }

  //
  // Create the root trees
  treeCollection=new CollectionTree(coch);
  treeEventTree=new EventTree(evch);
  treeTruthTree=new TruthTree(trch);

  //
  // and make them friends
  coch->AddFriend(evch,"AliasForEV0");
  coch->AddFriend(trch,"AliasForTruth0");

//   //
//   // disable a bunch of branches here if you feel like it
  // better not disable anything in coch. That's so small anyway.
//   evch->SetBranchStatus("*",0);
//   trch->SetBranchStatus("*",0);
//   // and now enable a select few branches
//   evch->SetBranchStatus("El_E",1);
//   evch->SetBranchStatus("El_N",1);
//   evch->SetBranchStatus("El_p_T",1);
//   evch->SetBranchStatus("El_eta",1);
//   evch->SetBranchStatus("El_phi",1);
//   evch->SetBranchStatus("El_EoverP",1);
//   evch->SetBranchStatus("El_charge",1);
//   //evch->SetBranchStatus("",1);
//   evch->SetBranchStatus("PJet_N",1);


  //
  // open the output histogram file
  OpenHistogramFile(cwd+histoFileName);

  //
  // now start parsing the configuration file
  //
  // debug level
  if(_config.Defined("DebugLevel")) {
    SetDebugLevel(_config.GetValue("DebugLevel",0));
    if(GetDebugLevel()>0) cout<<"<driver> Parsing configuration file "<<endl;
    if(GetDebugLevel()>0) cout<<"Setting debug level to "<<GetDebugLevel()<<endl;
  }
  //
  // the target top mass used in some optimization (default: 175GeV)
  SetTargetTopMass(_config.GetValue("MTop",175.));
  if(GetDebugLevel()>0) cout<<"Setting target top mass to "<<GetTargetTopMass()<<endl;
  //
  // include a file containing cuts
  if(_config.Defined("Include.CutsFile")) {
    TString cutsFile=cwd + _config.GetValue("Include.CutsFile","NONE");
    // check if this input file exists!
    FILE *ifp;
    if(!(ifp = fopen(cutsFile.Data(), "r"))) {
      cout<<"ERROR: cuts file "<<cutsFile.Data()<<" does not exist!"<<endl;
      return -1;
    }
    fclose(ifp);
    // the file exists, read it in:
    if(GetDebugLevel()>0) cout<<"Reading cuts config file: "<<cutsFile.Data()<<endl;    
    _config.ReadFile(cutsFile.Data(),kEnvChange);
  }
  //
  // also include a file containing weights for each source
  if(_config.Defined("Include.WeightsFile")) {
    TString weightsFile=cwd + _config.GetValue("Include.WeightsFile","NONE");
    // check if this input file exists!
    FILE *ifp;
    if(!(ifp = fopen(weightsFile.Data(), "r"))) {
      cout<<"ERROR: weights file "<<weightsFile.Data()<<" does not exist!"<<endl;
      return -1;
    }
    fclose(ifp);
    // the file exists, read it in:
    if(GetDebugLevel()>0) cout<<"Reading weights config file: "<<weightsFile.Data()<<endl;    
    _config.ReadFile(weightsFile.Data(),kEnvChange);
  }
  

  //
  // if we want to skim, then open a new file and create a clone of the 
  // old trees in the new file.
  if(_doSkim && _skimFileName!="NONE") {
    OpenSkimFile(cwd+_skimFileName);
    _skimCoTr=(TTree*) coch->CloneTree(0);
    _skimEvTr=(TTree*) evch->CloneTree(0);
    _skimTrTr=(TTree*) trch->CloneTree(0);
  }
  // 
  // the chains were read in successfully. Now initialize the analysis classes
  Particles::initialize(treeCollection,treeEventTree,treeTruthTree);
  Particles::SetEventCount(evtOffSet);
  return 0;
}
